module Philosophers where

import Control.Concurrent
import System.Random
import Control.Monad
import System.Environment(getArgs)


-- Time to eat and think (in ms)
eatThink :: Int
eatThink = 100 

-- A fork is an MVar -- empty if on the table, filled if in use
type Fork = MVar ()

-- Delay a randum amount up to `d` ms
delay :: Int-> IO ()
delay d = do 
   c<- randomRIO(0, d) -- Delay in ms
   threadDelay (c*1000) 

philosopher :: String -> Fork-> Fork -> IO ()
philosopher name left right = forever $ do
  putStrLn $ name ++ " is thinking."
  delay eatThink 
  putStrLn $ name ++ " is hungry."

  -- Take the forks  
  putMVar left ()
  delay 50 -- provokes a deadlock
  putMVar right ()

  putStrLn $ name ++ " is eating."
  delay eatThink
  putStrLn $ name ++ " finished eating, goes back to thinking." 
  
  -- Put down the forks
  takeMVar left
  takeMVar right

run :: Int -> IO ()
run n = do
  gl <- newEmptyMVar 
  forks <- replicateM n newEmptyMVar
  forM_ [0..n-1] $ \i-> 
     forkIO (philosopher ("Philosopher #"++ show i) 
                         (forks !! i)
                         (forks !! ((i+1) `mod` n))) 
  getLine
  return ()

main :: IO ()
main = do
  args <- getArgs
  case args of []  -> run 5
               x:_ -> run $ read x
