# Author: Jona Dirks <dirks2@uni-bremen.de>

import threading
import time
import random
from termcolor import colored


min_eating = 0
max_eating = 5

min_thinking = 2
max_thinking = 5


states = [0]*5
last_update = ""
print_lock = threading.Lock()

def print_state():
	with print_lock:
		print()
		print(last_update)
		print("-"*(11*5))
		print(" | ", end="")
		for id,p in enumerate(states):
			if p == 0:
				print("%d:  (  ) | "%id, end="")
			elif p == 1:
				print("%d:       | "%id, end="")
			elif p == 2:
				print(colored("%d:  (  ) | "%id, "red"), end="")
			elif p == 3:
				print(colored("%d:  (  ) | "%id, "red"), end="")
		print()
		print(" | ", end="")
		for p in states:
			if p == 0:
				print("  .°     | ", end="")
			elif p == 1:
				print("         | ", end="")
			elif p == 2:
				print(colored("  .°     | ", "red"), end="")
			elif p == 3:
				print(colored("  .°     | ", "red"), end="")
		print()
		print(" | ", end="")
		for p in states:
			if p == 0:
				print(" O       | ", end="")
			elif p == 1:
				print(" O       | ", end="")
			elif p == 2:
				print(colored(" O       | ", "red"), end="")
			elif p == 3:
				print(colored(" O       | ", "red"), end="")
		print()
		print(" | ", end="")
		for p in states:
			if p == 0:
				print("-H-      | ", end="")
			elif p == 1:
				print("`H´      | ", end="")
			elif p == 2:
				print(colored("-H´      | ", "red"), end="")
			elif p == 3:
				print(colored("`H-      | ", "red"), end="")
		print()
		print("-"*(11*5))


def eating(id):
	global last_update
	states[id] = 1
	last_update = "philosoph %d is eating"% id
	print_state()

def thinking(id):
	global last_update
	if states[id] != 2:
		states[id] = 0
		last_update = "philosoph %d is thinking"% id
		print_state()


def try_but_left(id):
	global last_update
	states[id] = 2
	last_update = "philosoph %d tryed to eat but the left fork was in use"% id
	print_state()

def try_but_right(id):
	global last_update
	states[id] = 3
	last_update = "philosoph %d tryed to eat but the right fork was in use"% id
	print_state()


class Fork:
	def __init__(self):
		self.takenby = -1
		self.lock = threading.Lock()

	def take_if_possible(self, id):
		with self.lock:
			if self.takenby == -1 or self.takenby == id:
				self.takenby=id
				return True
			else:
				return False

	def lay_down(self, id):
		if self.takenby==id:
			self.takenby=-1
		else:
			raise ValueError("Try to lay down a fork you don't have")
	

def philosoph(id, left_fork, right_fork):
	global states
	while True:
		thinking(id)
		time.sleep(random.randrange(min_eating, max_eating))
		if left_fork.take_if_possible(id):
			if right_fork.take_if_possible(id):
				eating(id)
				time.sleep(random.randrange(min_thinking, max_thinking))
				left_fork.lay_down(id)
				right_fork.lay_down(id)
			else:
				left_fork.lay_down(id)
				try_but_right(id)
		else:
			try_but_left(id)



fork0 = Fork()
fork1 = Fork()
fork2 = Fork()
fork3 = Fork()
fork4 = Fork()

philosoph0 = threading.Thread(target=philosoph, args=(0,fork0, fork1))
philosoph0.start()

philosoph1 = threading.Thread(target=philosoph, args=(1,fork1, fork2))
philosoph1.start()

philosoph2 = threading.Thread(target=philosoph, args=(2,fork2, fork3))
philosoph2.start()

philosoph3 = threading.Thread(target=philosoph, args=(3,fork3, fork4))
philosoph3.start()

philosoph4 = threading.Thread(target=philosoph, args=(4,fork4, fork0))
philosoph4.start()
